﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace BleedOut
{
    public class TlsHandshakeRecord : TlsRecord
    {
        public HandshakeRecordType HandshakeType { get; set; }

        public uint HandshakeRecordLength { get; set; }

        public TlsVersion HandshakeRecordVersion { get; set; }

        public TlsRandom Random { get; set; }

        public byte SessionIdLength { get; set; }

        public ushort CipherSuitesLength { get; set; }

        public CipherSuite[] CipherSuites { get; set; }

        public byte CompressionMethodsLength { get; set; }

        public byte[] CompressionMethods { get; set; }

        public ushort ExtensionsLength { get; set; }

        public TlsExtension[] Extensions { get; set; }

        public TlsHandshakeRecord()
        {
            ContentType = TlsRecordType.Handshake;
        }

        public override void Write(BinaryWriter writer)
        {
            base.Write(writer);

            writer.Write((byte)HandshakeType);

            var handshakeRecordLengthBytes = BigEndianBitConverter.GetBytes(HandshakeRecordLength);

            if (handshakeRecordLengthBytes[0] != 0)
            {
                throw new InvalidOperationException();
            }

            writer.Write(handshakeRecordLengthBytes[1]);
            writer.Write(handshakeRecordLengthBytes[2]);
            writer.Write(handshakeRecordLengthBytes[3]);
            writer.Write((ushort)HandshakeRecordVersion);
            Random.Write(writer);
            writer.Write(SessionIdLength);
            writer.Write(CipherSuitesLength);

            foreach (var c in CipherSuites)
            {
                writer.Write((ushort)c);
            }

            writer.Write(CompressionMethodsLength);
            writer.Write(CompressionMethods);
            writer.Write(ExtensionsLength);

            foreach (var e in Extensions)
            {
                e.Write(writer);
            }
        }

        private int ComputeExtensionsLength()
        {
            return Extensions
                .Select(x => sizeof(TlsExtensionType) + sizeof(ushort) + x.ComputeLength())
                .Sum();
        }

        public override int ComputeLength()
        {
            return
                sizeof(HandshakeRecordType) +
                (sizeof(uint) - 1) +
                ComputeDerivedLength();
        }

        public override int ComputeDerivedLength()
        {
            return
                sizeof(TlsVersion) +
                Random.ComputeLength() +
                sizeof(byte) +
                sizeof(ushort) +
                (sizeof(CipherSuite) * CipherSuites.Length) +
                sizeof(byte) +
                CompressionMethods.Length +
                sizeof(ushort) +
                ComputeExtensionsLength();
        }

        public override void SetDerivedLength()
        {
            SessionIdLength = 0;
            CipherSuitesLength = (ushort)(CipherSuites.Length * sizeof(CipherSuite));
            CompressionMethodsLength = (byte)CompressionMethods.Length;
            HandshakeRecordLength = (ushort)ComputeDerivedLength();
            ExtensionsLength = (ushort)ComputeExtensionsLength();
        }
    }
}
