﻿using System;
using System.IO;

namespace BleedOut
{
    public abstract class TlsRecord : IBinaryWritable, IComputableLength
    {
        public TlsRecordType ContentType { get; set; }

        public TlsVersion Version { get; set; }

        public ushort Length { get; set; }

        public virtual void Write(BinaryWriter writer)
        {
            writer.Write((byte)ContentType);
            writer.Write((ushort)Version);
            writer.Write(Length);
        }

        public abstract int ComputeLength();

        public void SetLength()
        {
            Length = (ushort)ComputeLength();
            SetDerivedLength();
        }

        public abstract int ComputeDerivedLength();

        public abstract void SetDerivedLength();

        public byte[] ToBytes()
        {
            using (var stream = new MemoryStream())
            {
                using (var writer = new BigEndianBinaryWriter(stream))
                {
                    Write(writer);
                    return stream.ToArray();
                }
            }
        }
    }
}
